from datetime import datetime
from typing import List, Optional

from pydantic import BaseModel

from py_ocsf_models.objects.kb_article import KBArticle
from py_ocsf_models.objects.remediation import Remediation


class VulnerabilityDetails(BaseModel):
    """
    The vulnerability is an unintended characteristic of a computing component or system configuration that multiplies the risk of an adverse event or a loss occurring either due to accidental exposure, deliberate attack, or conflict with new system components.

    Attributes:
    - Affected Code (affected_code) [Optional]: List of AffectedCode objects that describe details about code blocks identified as vulnerable.
    - Affected Software Packages (affected_packages) [Optional]: List of software packages identified as affected by a vulnerability/vulnerabilities.
    - CVE (cve) [Recommended]: The Common Vulnerabilities and Exposures (CVE).
    - CWE (cwe) [Recommended]: The CWE object represents a weakness in a software system that can be exploited by a threat actor to perform an attack.
    - Description (desc) [Optional]: The description of the vulnerability.
    - Exploit Availability (is_exploit_available) [Optional]: Indicates if an exploit or a PoC (proof-of-concept) is available for the reported vulnerability.
    - First Seen (first_seen_time) [Optional]: The time when the vulnerability was first observed.
    - Knowledgebase Articles (kb_article_list) [Optional]: A list of KB articles or patches related to an endpoint. A KB Article contains metadata that describes the patch or an update.
    - Last Seen (last_seen_time) [Optional]: The time when the vulnerability was most recently observed.
    - References (references) [Recommended]: A list of reference URLs with additional information about the vulnerability.
    - Related Vulnerabilities (related_vulnerabilities) [Optional]: List of vulnerabilities that are related to this vulnerability.
    - Remediation Guidance (remediation) [Optional]: The remediation recommendations on how to mitigate the identified vulnerability.
    - Severity (severity) [Optional]: The vendor assigned severity of the vulnerability.
    - Title (title) [Optional]: A title or a brief phrase summarizing the discovered vulnerability.
    - Vendor Name (vendor_name) [Optional]: The name of the vendor that identified the vulnerability.
    """

    # TODO
    # affected_code: Optional[List[AffectedCode]]
    # affected_packages: Optional[List[AffectedSoftwarePackage]]
    # cve: Optional[CVE]
    # cwe: Optional[CWE]
    desc: Optional[str]
    is_exploit_available: Optional[bool]
    first_seen_time: Optional[datetime]
    kb_article_list: Optional[List[KBArticle]]
    last_seen_time: Optional[datetime]
    references: Optional[List[str]]
    related_vulnerabilities: Optional[List[str]]
    remediation: Optional[Remediation]
    severity: Optional[str]
    title: Optional[str]
    vendor_name: Optional[str]
